/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.actions.SaveAllProjectsAction;
import com.eviware.soapui.config.SoapuiWorkspaceDocumentConfig;
import com.eviware.soapui.config.WorkspaceProjectConfig;
import com.eviware.soapui.impl.actions.ImportWsdlProjectAction;
import com.eviware.soapui.impl.actions.NewWsdlProjectAction;
import com.eviware.soapui.impl.settings.XmlBeansSettingsImpl;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.model.workspace.WorkspaceListener;
import com.eviware.soapui.support.SoapUIException;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * Default Workspace implementation
 * 
 * @author Ole.Matzura
 */

public class WorkspaceImpl extends AbstractModelItem implements Workspace
{
	private final static Logger log = Logger.getLogger( WorkspaceImpl.class );
	private List<Project> projectList = new ArrayList<Project>();
	private SoapuiWorkspaceDocumentConfig workspaceConfig;
	private String path = null;
	private Set<WorkspaceListener> listeners = new HashSet<WorkspaceListener>();
	private ImageIcon workspaceIcon;
	private XmlBeansSettingsImpl settings;

	public WorkspaceImpl( String path ) throws XmlException, IOException
	{
		this.path = path;

		File file = new File( path );
		if( file.exists() )
		{
			log.info( "Loading workspace from [" + file.getAbsolutePath() + "]" );
			workspaceConfig = SoapuiWorkspaceDocumentConfig.Factory.parse( file );
			if( workspaceConfig.getSoapuiWorkspace().getSettings() == null )
				workspaceConfig.getSoapuiWorkspace().addNewSettings();

			settings = new XmlBeansSettingsImpl( this, SoapUI.getSettings(), workspaceConfig
						.getSoapuiWorkspace().getSettings() );

			List<WorkspaceProjectConfig> projects = workspaceConfig.getSoapuiWorkspace()
						.getProjectList();
			for( int i = 0; i < projects.size(); i++ )
			{
				WorkspaceProjectConfig wsc = projects.get( i );
				String str = wsc.getStringValue();

				if( new File( str ).exists() )
				{
					try
					{
						WsdlProject project = new WsdlProject( str, this );
						projectList.add( project );
					}
					catch( Exception e )
					{
						UISupport.showErrorMessage( "Failed to load project [" + str
									+ "]\nfrom workspace; " + e.getMessage() );

						e.printStackTrace();
					}
				}
				else
				{
					UISupport.showErrorMessage( "project [" + str
								+ "]\nnot found, disabling in workspace" );
					
					try
					{
						WsdlProject project = new WsdlProject( str, this, false );
						projectList.add( project );
					}
					catch( Exception e )
					{
						e.printStackTrace();
					}
				}
			}
		}
		else
		{
			workspaceConfig = SoapuiWorkspaceDocumentConfig.Factory.newInstance();
			workspaceConfig.addNewSoapuiWorkspace().setName( "Default Workspace" );
			workspaceConfig.getSoapuiWorkspace().addNewSettings();

			settings = new XmlBeansSettingsImpl( this, SoapUI.getSettings(), workspaceConfig
						.getSoapuiWorkspace().getSettings() );
		}

		initActions();

		workspaceIcon = UISupport.createImageIcon( "/workspace.gif" );
	}

	public void setPath( String path )
	{
		this.path = path;
	}

	public Project[] getProjects()
	{
		return projectList.toArray( new Project[projectList.size()] );
	}

	public void setName( String name )
	{
		workspaceConfig.getSoapuiWorkspace().setName( name );
	}

	public String getName()
	{
		return "Projects";
	}

	public Project getProjectAt( int index )
	{
		return projectList.get( index );
	}

	public Project getProjectByName( String projectName )
	{
		for( Project project : projectList )
		{
			if( project.getName().equals( projectName ) )
				return project;
		}

		return null;
	}

	public int getProjectCount()
	{
		return projectList.size();
	}

	public void onClose()
	{
		save( false );
	}

	public void save( boolean workspaceOnly )
	{
		try
		{
			List<WorkspaceProjectConfig> projects = new ArrayList<WorkspaceProjectConfig>();

			// save projects first
			for( int c = 0; c < getProjectCount(); c++ )
			{
				WsdlProject project = ( WsdlProject ) getProjectAt( c );

				if( !workspaceOnly )
					project.save();

				String path = project.getPath();
				if( path != null )
				{
					WorkspaceProjectConfig wpc = WorkspaceProjectConfig.Factory.newInstance();
					wpc.setStringValue( path );
					projects.add( wpc );
				}
			}

			if( path == null )
			{
				File file = UISupport.getFileDialogs().saveAs( this, "Save workspace", ".xml",
							"XML Files (*.xml)", null );
				if( file == null )
					return;

				path = file.getAbsolutePath();
			}

			workspaceConfig.getSoapuiWorkspace().setProjectArray(
						projects.toArray( new WorkspaceProjectConfig[projects.size()] ) );
			workspaceConfig.getSoapuiWorkspace().setSoapuiVersion( SoapUI.SOAPUI_VERSION );

			File workspaceFile = new File( path );
			workspaceConfig.save( workspaceFile );

			log.info( "Saved workspace to [" + workspaceFile.getAbsolutePath() + "]" );
		}
		catch( IOException e )
		{
			log.error( "Failed to save workspace: " + e.getMessage(), e );
		}
	}

	private void initActions()
	{
		addAction( new NewWsdlProjectAction( this ) );
		addAction( new ImportWsdlProjectAction( this ) );
		addAction( ActionSupport.SEPARATOR_ACTION );
		addAction( new SaveAllProjectsAction( this ) );
		addAction( ActionSupport.SEPARATOR_ACTION );
		addAction( new ShowOnlineHelpAction( HelpUrls.WORKSPACE_HELP_URL ) );
	}

	public void addWorkspaceListener( WorkspaceListener listener )
	{
		listeners.add( listener );
	}

	public void removeWorkspaceListener( WorkspaceListener listener )
	{
		listeners.remove( listener );
	}

	public Project importProject( String fileName ) throws SoapUIException
	{
		File projectFile = new File( fileName );

		WsdlProject project = new WsdlProject( projectFile.getAbsolutePath(), this );
		projectList.add( project );

		for( Iterator<WorkspaceListener> iter = listeners.iterator(); iter.hasNext(); )
		{
			WorkspaceListener listener = iter.next();
			listener.projectAdded( project );
		}

		save( true );

		return project;
	}

	public WsdlProject createProject( String name ) throws SoapUIException
	{
		File projectFile = new File( createProjectFileName( name ) );
		File file = UISupport.getFileDialogs().saveAs( this, "Create Project", ".xml",
					"XML Files (*.xml)", projectFile );
		if( file == null )
			return null;

		return ( WsdlProject ) createProject( name, file );
	}

	public Project createProject( String name, File file ) throws SoapUIException
	{
		File projectFile = file;
		while( projectFile.exists() )
		{
			Boolean result = Boolean.FALSE;
			while( !result.booleanValue() )
			{
				result = UISupport.confirmOrCancel( "File exists, overwrite?", "Overwrite project?" );
				if( result == null )
					return null;
				if( result.booleanValue() )
				{
					projectFile.delete();
				}
				else
				{
					projectFile = UISupport.getFileDialogs().saveAs( this, "Create Project", ".xml",
								"XML Files (*.xml)", projectFile );
					if( projectFile == null )
						return null;
					else
						break;
				}
			}
		}

		WsdlProject project = new WsdlProject( projectFile.getAbsolutePath(), this );

		project.setName( name );
		projectList.add( project );

		for( Iterator<WorkspaceListener> iter = listeners.iterator(); iter.hasNext(); )
		{
			WorkspaceListener listener = iter.next();
			listener.projectAdded( project );
		}

		try
		{
			project.save();
		}
		catch( IOException e )
		{
			log.error( "Failed to save project: " + e.getMessage(), e );
		}
		save( true );

		return project;
	}

	private String createProjectFileName( String name )
	{
		StringBuffer result = new StringBuffer();
/*		for( int c = 0; c < name.length(); c++ )
		{
			char ch = name.charAt( c );
			if( Character.isLetterOrDigit( ch ) )
			{
				result.append( ch );
			}
		}
*/
		result.append( name );
		result.append( "-soapui-project.xml" );
		return result.toString();
	}

	public void removeProject( Project project )
	{
		int ix = projectList.indexOf( project );
		if( ix == -1 )
			throw new RuntimeException( "Project [" + project.getName()
						+ "] not available in workspace for removal" );

		projectList.remove( ix );

		try
		{
			fireProjectRemoved( project );
		}
		finally
		{
			project.release();
			workspaceConfig.getSoapuiWorkspace().removeProject( ix );
		}
	}
	
	public void reloadProject( WsdlProject project ) throws SoapUIException
	{
		int ix = projectList.indexOf( project );
		if( ix == -1 )
			throw new RuntimeException( "Project [" + project.getName()
						+ "] not available in workspace for removal" );

		projectList.remove( ix );
		fireProjectRemoved( project );
		
		project.release();
		project = new WsdlProject( project.getPath(), this );
		
		projectList.add( ix, project );

		for( Iterator<WorkspaceListener> iter = listeners.iterator(); iter.hasNext(); )
		{
			WorkspaceListener listener = iter.next();
			listener.projectAdded( project );
		}

		workspaceConfig.getSoapuiWorkspace().getProjectArray( ix ).setStringValue( project.getPath() );
		save( true );
	}

	private void fireProjectRemoved( Project project )
	{
		WorkspaceListener[] listenerArray = listeners
					.toArray( new WorkspaceListener[listeners.size()] );
		for( int c = 0; c < listenerArray.length; c++ )
		{
			listenerArray[c].projectRemoved( project );
		}
	}

	public ImageIcon getIcon()
	{
		return workspaceIcon;
	}

	public Settings getSettings()
	{
		return settings;
	}

	public int getIndexOfProject( Project project )
	{
		return projectList.indexOf( project );
	}

	public String getPath()
	{
		return path;
	}

	public void release()
	{
		settings.release();

		for( Project project : projectList )
			project.release();
	}

	public int hashCode()
	{
		return( this.getPath().hashCode() + this.getName().hashCode() );
	}

	public boolean equals( Object o )
	{
		if( this == o )
			return true;
		if( !( o instanceof Workspace ) )
			return false;
		Workspace workspace = ( Workspace ) o;
		return workspace.hashCode() == this.hashCode();
	}

}
