/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

import com.eviware.soapui.config.DefinitionCacheConfig;
import com.eviware.soapui.config.EndpointConfig;
import com.eviware.soapui.config.EndpointsConfig;
import com.eviware.soapui.config.InterfaceConfig;
import com.eviware.soapui.config.OperationConfig;
import com.eviware.soapui.config.SoapVersionTypesConfig;
import com.eviware.soapui.impl.actions.ShowDesktopPanelAction;
import com.eviware.soapui.impl.wsdl.actions.iface.ExportDefinitionAction;
import com.eviware.soapui.impl.wsdl.actions.iface.GenerateMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.iface.GenerateTestSuiteAction;
import com.eviware.soapui.impl.wsdl.actions.iface.InterfaceEndpointsAction;
import com.eviware.soapui.impl.wsdl.actions.iface.RemoveInterfaceAction;
import com.eviware.soapui.impl.wsdl.actions.iface.UpdateInterfaceAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.axis1.Axis1XWSDL2JavaAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.axis2.Axis2WSDL2CodeAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.dotnet.DotNetWsdlAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.gsoap.GSoapAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.jaxb.JaxbXjcAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.jbossws.JBossWSConsumeAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.jbossws.WSToolsRegenerateJava2WsdlAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.jbossws.WSToolsWsdl2JavaAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.oracle.OracleWsaGenProxyAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.tcpmon.TcpMonAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.wscompile.WSCompileAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.wsi.WSIAnalyzeAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.wsimport.WSImportAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.xfire.XFireAction;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.xmlbeans.XmlBeans2Action;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.soap.SoapMessageBuilder;
import com.eviware.soapui.impl.wsdl.support.soap.SoapVersion;
import com.eviware.soapui.impl.wsdl.support.wsdl.CachedWsdlLoader;
import com.eviware.soapui.impl.wsdl.support.wsdl.UrlWsdlLoader;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlLoader;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.InterfaceListener;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionExtensionPoint;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.action.DefaultActionList;
import com.eviware.soapui.support.types.StringList;

/**
 * WSDL implementation of Interface, maps to a WSDL Binding
 * 
 * @author Ole.Matzura
 */

public class WsdlInterface extends AbstractWsdlModelItem<InterfaceConfig> implements Interface
{
	public static final String STYLE_DOCUMENT = "Document";
	public static final String STYLE_RPC = "RPC";

   public static final String JBOSSWS_ACTIONS = "jbossws";
   public static final String WSTOOLS_ACTIONS = "wstools";
   public static final String XML_ACTIONS = "xml";

	private final static Logger log = Logger.getLogger( WsdlInterface.class );
	
   private List<WsdlOperation> operations = new ArrayList<WsdlOperation>();
   private WsdlProject project;
   private SoapMessageBuilder soapMessageBuilder;
   private WsdlContext wsdlContext;
   private Set<InterfaceListener> listeners = new HashSet<InterfaceListener>();
   private DefaultActionList generateActions;
   
   public WsdlInterface( WsdlProject project, InterfaceConfig interfaceConfig )
   {
   	super( interfaceConfig, project, "/interface2.gif" );
   	
      this.project = project;
      
      if( interfaceConfig.getEndpoints() == null )
         interfaceConfig.addNewEndpoints();
      
      List<OperationConfig> operationConfigs = interfaceConfig.getOperationList();
      for (int i = 0; i < operationConfigs.size(); i++)
      {
         operations.add( new WsdlOperation( this, operationConfigs.get(i) ));
      }
      
      buildActions();
   }

	private void buildActions()
	{
		generateActions = new DefaultActionList( "Generate" );
      generateActions.addAction( new ActionExtensionPoint( this, WSTOOLS_ACTIONS ) );
		generateActions.addAction( new WSToolsWsdl2JavaAction( this ));
		generateActions.addAction( new JBossWSConsumeAction( this ));
      generateActions.addAction( ActionSupport.SEPARATOR_ACTION );
      generateActions.addAction( new ActionExtensionPoint( this, JBOSSWS_ACTIONS ) );
      generateActions.addAction( new WSCompileAction( this ));
      generateActions.addAction( new WSImportAction( this ));
      generateActions.addAction( ActionSupport.SEPARATOR_ACTION );
      generateActions.addAction( new Axis1XWSDL2JavaAction( this ));
      generateActions.addAction( new Axis2WSDL2CodeAction( this ));
      generateActions.addAction( new XFireAction( this ));
      generateActions.addAction( new OracleWsaGenProxyAction( this ));
      generateActions.addAction( ActionSupport.SEPARATOR_ACTION );
      generateActions.addAction( new ActionExtensionPoint( this, XML_ACTIONS ) );
      generateActions.addAction( new JaxbXjcAction( this ));
      generateActions.addAction( new XmlBeans2Action( this ));
      generateActions.addAction( ActionSupport.SEPARATOR_ACTION );
      generateActions.addAction( new DotNetWsdlAction( this ));
      generateActions.addAction( new GSoapAction( this ));
      
      addAction( new ShowDesktopPanelAction( "Open Interface Viewer", "Opens the Interface Viewer for this Interface", this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ActionSupport.ActionListAction( generateActions ));
      addAction( new WSToolsRegenerateJava2WsdlAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new WSIAnalyzeAction( this ));
      addAction( new TcpMonAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new GenerateTestSuiteAction( this ));
      addAction( new GenerateMockServiceAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new InterfaceEndpointsAction( this ) );
      addAction( new UpdateInterfaceAction( this ) );
      addAction( new ExportDefinitionAction( this ) );
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new RemoveInterfaceAction( this ) );
      addAction( new ShowOnlineHelpAction( HelpUrls.INTERFACE_HELP_URL ));
	}

   public DefaultActionList getGenerateActions()
   {
      return generateActions;
   }

	public String[] getEndpoints()
   {
      EndpointsConfig endpoints = getConfig().getEndpoints();
      List<EndpointConfig> endpointArray = endpoints.getEndpointList();
      
      String [] result = new String[endpointArray.size()];
      
      for( int c = 0; c < result.length; c++ )
      {
      	EndpointConfig endpoint = endpointArray.get(c);
      	if( endpoint.isSetLabel() )
      		result[c] = endpoint.getLabel();
      	else
      		result[c] = endpoint.getStringValue();
      }
      
		return result; 
   }

   public WsdlOperation getOperationAt(int index)
   {
      return operations.get(index);
   }

   public int getOperationCount()
   {
      return operations.size();
   }

   public WsdlOperation addNewOperation(BindingOperation operation)
   {
      WsdlOperation operationImpl = new WsdlOperation( this, getConfig().addNewOperation() );
      operations.add( operationImpl );

      operationImpl.initFromBindingOperation( operation, false );
      fireOperationAdded( operationImpl );
      return operationImpl;
   }

   public WsdlProject getProject()
   {
      return project;
   }

   public void addEndpoint(String endpoint)
   {
      if( endpoint == null || endpoint.trim().length() == 0 ) return;
      
      endpoint = endpoint.trim();
      String[] endpoints = getEndpoints();
      
      // dont add the same endpoint twice
      if( Arrays.asList( endpoints ).contains( endpoint )) return;
      
      getConfig().getEndpoints().addNewEndpoint().setStringValue(endpoint);
		
		notifyPropertyChanged(ENDPOINT_PROPERTY, null, endpoint);
   }
   
   public void changeEndpoint(String oldEndpoint, String newEndpoint)
	{
   	if( oldEndpoint == null || oldEndpoint.trim().length() == 0 ) return;
   	if( newEndpoint == null || newEndpoint.trim().length() == 0 ) return;
   	
   	EndpointsConfig endpoints = getConfig().getEndpoints();
      List<EndpointConfig> endpointArray = endpoints.getEndpointList();
      
      String [] result = new String[endpointArray.size()];
      
      for( int c = 0; c < result.length; c++ )
      {
      	EndpointConfig endpoint = endpointArray.get( c );
      	if( endpoint.getStringValue().equals( oldEndpoint ) )
      	{
      		endpoint.setStringValue( newEndpoint );
         	notifyPropertyChanged(ENDPOINT_PROPERTY, oldEndpoint, newEndpoint);
         	break;
      	}
      }
	}

	public void removeEndpoint(String endpoint)
	{
		EndpointsConfig endpoints = getConfig().getEndpoints();
      List<EndpointConfig> endpointArray = endpoints.getEndpointList();
      
      String [] result = new String[endpointArray.size()];
      
      for( int c = 0; c < result.length; c++ )
      {
      	EndpointConfig ep = endpointArray.get(c);
      	if( ep.getStringValue().equals( endpoint ) )
      	{
      		endpoints.removeEndpoint( c );
         	notifyPropertyChanged(ENDPOINT_PROPERTY, endpoint, null );
         	break;
      	}
      }
	}

	public void setDefinition( String wsdlUrl, boolean cache )
   {
      String old = getDefinition();
      
      getConfig().setDefinition( wsdlUrl );
      
      if( wsdlContext != null )
      {
      	wsdlContext.setDefinition( wsdlUrl, getConfig().getDefinitionCache() );
      	wsdlContext.setSoapVersion( getSoapVersion() );
      }
      
      notifyPropertyChanged( DEFINITION_PROPERTY, old, wsdlUrl );
   }

	public DefinitionCacheConfig cacheDefinition( WsdlLoader loader ) throws Exception
	{
		log.debug( "Caching definition for [" + loader.getBaseURI() + "]" );
      if( getConfig().isSetDefinitionCache() )
      	getConfig().unsetDefinitionCache();

		DefinitionCacheConfig definitionCache = getConfig().addNewDefinitionCache();
		definitionCache.set( WsdlLoader.cacheWsdl( loader  ) );
		return definitionCache;
	}
   
   public String getDefinition()
   {
      return getConfig().isSetDefinition() ? getConfig().getDefinition() : null;
   }
   
   public synchronized WsdlContext getWsdlContext()
   {
      if( wsdlContext == null )
      {
      	wsdlContext = new WsdlContext( getDefinition(), getSoapVersion(), getConfig().getDefinitionCache(), this );
      }  
      
      return wsdlContext;
   }
   
   /**
    * Used by importer so we dont need to reload the context after importing..
    * @param wsdlContext
    */
   
   public void setWsdlContext( WsdlContext wsdlContext )
   {
   	this.wsdlContext = wsdlContext;
   	this.wsdlContext.setSoapVersion( getSoapVersion() );
   	
   	if( !getConfig().isSetDefinitionCache() )
			getConfig().addNewDefinitionCache();
   	
   	if( wsdlContext.getCacheConfig() != null )
   	{
   		// use cache from context
   		getConfig().setDefinitionCache( wsdlContext.getCacheConfig() );
   	}
   }

   public SoapMessageBuilder getMessageBuilder()
   {
      if( soapMessageBuilder == null )
      {
         try
         {
            soapMessageBuilder = new SoapMessageBuilder( this );
         }
         catch (Exception e)
         {
            e.printStackTrace();
         }
      }
      return soapMessageBuilder;
   }

   public void setSoapMessageBuilder(SoapMessageBuilder builder)
   {
      soapMessageBuilder = builder;
      soapMessageBuilder.setInterface( this );
   }

   public QName getBindingName()
   {
      return getConfig().getBindingName() == null ? null : QName.valueOf(getConfig().getBindingName());
   }

   public void setBindingName(QName name)
   {
   	getConfig().setBindingName( name.toString() );
   }
   
   public SoapVersion getSoapVersion()
   {
   	if( getConfig().getSoapVersion() == SoapVersionTypesConfig.X_1_2  )
   		return SoapVersion.Soap12;
   	
   	return SoapVersion.Soap11;
   }
   
   public void setSoapVersion( SoapVersion version )
   {
   	if( version == SoapVersion.Soap11 )
   		getConfig().setSoapVersion( SoapVersionTypesConfig.X_1_1 );
   	else if( version == SoapVersion.Soap12 )
   		getConfig().setSoapVersion( SoapVersionTypesConfig.X_1_2 );
   	else 
   		throw new RuntimeException( "Unknown soapVersion [" + version + "], must be 1.1 or 1.2");
   	
   	getWsdlContext().setSoapVersion( version );
   }

	@SuppressWarnings("unchecked")
	public boolean updateDefinition(String url, boolean createRequests) throws Exception
	{
		WsdlContext newContext = null;
		
		if( getConfig().isSetDefinitionCache() )
			getConfig().unsetDefinitionCache();
		
		if( !getSettings().getBoolean( WsdlSettings.CACHE_WSDLS ) )
		{
			newContext = new WsdlContext( url, getSoapVersion(), null, null );
			newContext.load();
		}
		else
		{
			newContext = new WsdlContext( url, getSoapVersion(), null, this );	
		}
		
		Definition definition = newContext.getDefinition();
		Service service = null;
		Port port = null;
		Binding binding = null;
		
		// start by finding the old binding in the new definition
		Map serviceMap = definition.getAllServices();
		Iterator<String> i = serviceMap.keySet().iterator();
		while( i.hasNext() )
		{
			service = (Service) serviceMap.get( i.next() );
			Map portMap = service.getPorts();
			
			Iterator i2 = portMap.keySet().iterator();
			while( i2.hasNext() )
			{
				port = (Port) portMap.get( i2.next() );
				if( port.getBinding().getQName().equals( getBindingName() ))
				{
					binding = port.getBinding();
				}
			}
			
			if( binding != null ) break;
			service = null;
		}
		
		if( service == null && binding == null )
		{
			binding = definition.getBinding( getBindingName() );
		}

		// missing matching binding, prompt for new one to use instead (will happen if binding has been renamed)
		if( binding == null )
		{
			Map bindings = definition.getAllBindings();
			
			Object retval = UISupport.prompt(  
					"Missing matching binding [" + getBindingName() + "] in definition, select new\nbinding to map to", 
					"Map Binding", bindings.keySet().toArray() );
			
			if( retval == null )
				return false;
			
			binding = (Binding) bindings.get( retval );
			setBindingName( binding.getQName() );
		}
		
		// prepare for transfer of operations/requests
		List<BindingOperation> newOperations = new ArrayList<BindingOperation>( binding.getBindingOperations() );
		Map<String,WsdlOperation> oldOperations = new HashMap<String,WsdlOperation>();
		for( int c = 0; c < operations.size(); c++ )
			oldOperations.put( operations.get( c ).getBindingOperationName(), operations.get( c ) );
		
		// clear existing from both collections
		for( int c = 0; c < newOperations.size(); c++ )
		{
			BindingOperation newOperation = newOperations.get( c );
			String bindingOperationName = newOperation.getName();
			if( oldOperations.containsKey( bindingOperationName) )
			{
				log.info( "Synchronizing existing operation [" + bindingOperationName + "]" );
				WsdlOperation wsdlOperation = oldOperations.get( bindingOperationName );
				wsdlOperation.initFromBindingOperation( newOperation, true );
				
				oldOperations.remove( bindingOperationName );
				newOperations.remove( c );
				c--;
			}
		}
		
		//	remove leftover operations
		i = oldOperations.keySet().iterator();
		while( i.hasNext())
		{
			String name = i.next();
			
			if( newOperations.size() > 0 )
		   {
				List<String> list = new ArrayList<String>();
				list.add( "none - delete operation" );
				for( int c = 0; c < newOperations.size(); c++ )
					list.add( newOperations.get( c ).getName() );
				
				String retval = (String) UISupport.prompt( 
						"Binding operation [" + name + "] not found in new interface, select new\nbinding operation to map to", 
						"Map Operation", list.toArray(), "none - delete operation" );
				
				if( retval == null )
				{
				   UISupport.showErrorMessage( "Aborting update of interface" );
				   return false;
				}
				
				int ix = list.indexOf( retval)-1;
				
				// delete operation?
				if( ix < 0 )
				{
					deleteOperation( name );
				}
				// change operation?
				else
				{
					BindingOperation newOperation = newOperations.get( ix );
					WsdlOperation wsdlOperation = oldOperations.get( name );
					wsdlOperation.initFromBindingOperation( newOperation, true );
					newOperations.remove( ix );
				}

				oldOperations.remove( name );
		   }
			else 
			{
				deleteOperation( name );
				oldOperations.remove( name );
			}
			
			i = oldOperations.keySet().iterator();
		}
		
		wsdlContext = newContext;
		if( soapMessageBuilder != null )
			soapMessageBuilder.setWsdlContext( wsdlContext );
		
		// add leftover new operations
		if( newOperations.size() > 0 )
		{
			for( int c = 0; c < newOperations.size(); c++ )
			{
				BindingOperation newOperation = newOperations.get( c );
				WsdlOperation wsdlOperation = addNewOperation( newOperation );
				
				if( createRequests )
				{
					WsdlRequest request = wsdlOperation.addNewRequest( "Request 1");
               try
               {
                  request.setRequestContent( wsdlOperation.createRequest( true ));
               }
               catch (Exception e)
               {
                  e.printStackTrace();
               }
				}
			}
		}
		
    	setDefinition( url, false );

		if( port != null )
		{
			String endpoint = WsdlUtils.getSoapEndpoint( port );
	      if( endpoint != null )
	      {
	      	StringList list = new StringList( getEndpoints() );
	      	if( !list.contains( endpoint ))
	      	{
	      		if( UISupport.confirm( "Update existing requests with new endpoint\n[" + endpoint + "]", "Update Definition" ))
	      		{
	      			for( int c = 0; c < getOperationCount(); c++ )
	               {
	               	Operation operation = getOperationAt( c );

	               	for( int ix = 0; ix < operation.getRequestCount(); ix++ )
	               	{
	               		operation.getRequestAt( ix ).setEndpoint( endpoint );
	               	}
	               }
	      		}

	      		getConfig().getEndpoints().insertNewEndpoint( 0 ).setStringValue(endpoint);
		   		notifyPropertyChanged(ENDPOINT_PROPERTY, null, endpoint);
	      	}
	      }
		}
		
		return true;
	}

	private void deleteOperation(String bindingOperationName)
	{
		for( int c = 0; c < operations.size(); c++ )
		{
			WsdlOperation wsdlOperation = operations.get( c );
			if( wsdlOperation.getBindingOperationName().equals( bindingOperationName ))
			{
				log.info( "deleting operation [" + bindingOperationName + "]" );
				
				// remove requests first (should this be done by some listener?)
				while( wsdlOperation.getRequestCount() > 0 )
					wsdlOperation.removeRequest( (WsdlRequest) wsdlOperation.getRequestAt( 0 ));
				
				operations.remove( c );

				try
				{
					fireOperationRemoved( wsdlOperation );
				}
				finally
				{
					wsdlOperation.release();
					getConfig().removeOperation( c );
				}
				
				return;
			}
		}
	}
	
	public void fireOperationAdded( WsdlOperation operation )
   {
      InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].operationAdded( operation );
      }
   }
	
	public void fireOperationUpdated( WsdlOperation operation )
   {
      InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].operationUpdated( operation );
      }
   }
   
   public void fireOperationRemoved( WsdlOperation operation )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].operationRemoved( operation );
      }
   }
   
   public void fireRequestAdded( WsdlRequest request )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].requestAdded( request );
      }
   }
   
   public void fireRequestRemoved( WsdlRequest request )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].requestRemoved( request );
      }
   }

	public void addInterfaceListener(InterfaceListener listener)
	{
		listeners.add( listener );
	}

	public void removeInterfaceListener(InterfaceListener listener)
	{
		listeners.remove( listener );
	}

	/**
	 * Gets the endpoint url for the specified endpoint (which may be a label)
	 * 
	 * @param endpoint the endpoint to get for
	 * @return the endpoints url
	 */
	
	public String getEndpointURL(String endpoint)
	{
		List<EndpointConfig> endpointArray = getConfig().getEndpoints().getEndpointList();
		for( EndpointConfig endpointConfig : endpointArray )
		{
			if( endpointConfig.isSetLabel() && endpointConfig.getLabel().equals( endpoint ))
				return endpointConfig.getStringValue();
		}
		
		return endpoint;
	}

	public WsdlOperation getOperationByName(String name)
	{
		return (WsdlOperation) getWsdlModelItemByName( operations, name );
	}
	
	public boolean isCached()
	{
		return getConfig().isSetDefinitionCache();
	}

	public WsdlLoader createWsdlLoader()
	{
		return isCached() ? new CachedWsdlLoader( getConfig().getDefinitionCache() ) : 
			new UrlWsdlLoader( getDefinition() );
	}
	
	public void clearCache()
	{
		if( wsdlContext != null )
			wsdlContext.setDefinitionCache( null );
		
		if( getConfig().isSetDefinitionCache() )
			getConfig().unsetDefinitionCache();
	}
	
	public String getStyle()
	{
		if( wsdlContext == null || !wsdlContext.isLoaded() )
			return "<not loaded>";
		
		try
		{
			Binding binding = wsdlContext.getDefinition().getBinding( getBindingName() );
			if( binding == null )
				return "<missing binding>";
			
			if( WsdlUtils.isRpc( binding))
			{
				return STYLE_RPC;
			}
			else
			{
				return STYLE_DOCUMENT;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return "<error>";
		}
	}

	public void release()
	{
		super.release();
		
		for( WsdlOperation operation : operations )
			operation.release();
	}

	public List<Operation> getOperations()
	{
		return new ArrayList<Operation>( operations );
	}
}
