/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.wsdl.BindingOperation;
import javax.wsdl.Message;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.SchemaGlobalElement;
import org.apache.xmlbeans.SchemaType;

import com.eviware.soapui.config.AttachmentConfig;
import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.CredentialsConfig;
import com.eviware.soapui.impl.actions.ShowDesktopPanelAction;
import com.eviware.soapui.impl.wsdl.actions.operation.AddToMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.request.AddRequestToTestCaseAction;
import com.eviware.soapui.impl.wsdl.actions.request.CloneRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.DeleteRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.RenameRequestAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.submit.RequestTransportRegistry;
import com.eviware.soapui.impl.wsdl.submit.filters.PropertyExpansionRequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.AttachmentUtils;
import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.impl.wsdl.support.FileAttachment;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.ModelItemIconAnimator;
import com.eviware.soapui.impl.wsdl.support.RequestFileAttachment;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils.SoapHeader;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.MessagePart;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.iface.SubmitListener;
import com.eviware.soapui.model.support.InterfaceListenerAdapter;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.types.StringToStringMap;

/**
 * Request implementation holding a SOAP request
 * 
 * @author Ole.Matzura
 */

public class WsdlRequest extends AbstractWsdlModelItem<CallConfig> implements Request, AttachmentContainer
{
	public static final String RESPONSE_PROPERTY = WsdlRequest.class.getName() + "@response";
	public static final String RESPONSE_CONTENT_PROPERTY = WsdlRequest.class.getName() + "@response-content";
	public static final String ATTACHMENTS_PROPERTY = WsdlRequest.class.getName() + "@attachments";
	public static final String INLINE_RESPONSE_ATTACHMENTS = WsdlRequest.class.getName() + "@inline-response-attachments";
	public static final String EXPAND_MTOM_RESPONSE_ATTACHMENTS = WsdlRequest.class.getName() + "@expand-mtom-attachments";
	public static final String STRIP_WHITESPACES = WsdlRequest.class.getName() + "@strip-whitespaces";
	public static final String REQUEST_HEADERS_PROPERTY = WsdlRequest.class.getName() + "@request-headers";
	public static final String DISABLE_MULTIPART_ATTACHMENTS = WsdlRequest.class.getName() + "@disable-multipart-attachments";
	
	public final static String PW_TYPE_NONE="None";
   public final static String PW_TYPE_DIGEST="PasswordDigest";
   public final static String PW_TYPE_TEXT="PasswordText";
   
   private final WsdlOperation operation;
   private WsdlResponse response;
   private final static Log log = LogFactory.getLog( WsdlRequest.class );
	

   protected List<FileAttachment> attachments = new ArrayList<FileAttachment>();
   
	private RequestIconAnimator iconAnimator;
   private Set<SubmitListener> listeners = new HashSet<SubmitListener>();
	private List<WsdlAttachmentPart> definedAttachmentParts;
	
	private InternalInterfaceListener interfaceListener = new InternalInterfaceListener();

   public WsdlRequest( WsdlOperation operation, CallConfig callConfig )
   {
   	super( callConfig, operation, null );
   	
      this.operation = operation;
      
      initActions();
      initEndpoints();
      initAttachments();
      
      iconAnimator = initIconAnimator();
      
      // ensure encoding
      if( callConfig.getEncoding() == null || callConfig.getEncoding().length() == 0 )
      {
      	callConfig.setEncoding( "UTF-8" );
      }
      
      addSubmitListener( iconAnimator );
    
      operation.getInterface().addPropertyChangeListener( interfaceListener );
      operation.getInterface().addInterfaceListener( interfaceListener );
   }
   
   private void initAttachments()
	{
		for( AttachmentConfig ac : getConfig().getAttachmentList() )
		{
			FileAttachment attachment = new RequestFileAttachment( ac, this );
			attachments.add( attachment);
		}
	}

	public void updateConfig(CallConfig request)
	{
		setConfig( request );
	}
   
   public ModelItemIconAnimator getIconAnimator()
   {
   	return iconAnimator;
   }

   protected RequestIconAnimator initIconAnimator()
   {
   	return new RequestIconAnimator();
   }

   protected void initEndpoints()
   {
   	if( getEndpoint() == null )
   	{
	      String[] endpoints = operation.getInterface().getEndpoints();
	      if( endpoints.length > 0 )
	      {
	         setEndpoint( endpoints[0] );
	      }
   	}
   }

   protected void initActions()
   {
      addAction( new ShowDesktopPanelAction( "Open Request Editor", "Opens the Request Editor for this request", this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new AddRequestToTestCaseAction( this ));
//      addAction( new AddAsMockResponseStepAction( this ));
      addAction( new AddToMockServiceAction( this ));
      addAction( new CloneRequestAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new RenameRequestAction( this ));
      addAction( new DeleteRequestAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.REQUEST_HELP_URL ));
   }

   public String getRequestContent()
   {
     return getConfig().getRequest();
   }
   
   public void setEndpoint(String endpoint)
   {
      String old = getEndpoint();
      if( old != null && old.equals( endpoint ))
      	return;
      
      getConfig().setEndpoint( endpoint );
      notifyPropertyChanged( ENDPOINT_PROPERTY, old, endpoint);
   }

   public String getEndpoint()
   {
      return getConfig().getEndpoint();
   }

   public String getEncoding()
   {
      return getConfig().getEncoding();
   }

   public void setEncoding(String encoding)
   {
      String old = getEncoding();
      getConfig().setEncoding( encoding );
      notifyPropertyChanged( ENCODING_PROPERTY, old, encoding );
   }
   
   public StringToStringMap getRequestHeaders()
   {
   	return StringToStringMap.fromXml( getSettings().getString( REQUEST_HEADERS_PROPERTY, null ));
   }
   
   public void setRequestHeaders( StringToStringMap map )
   {
   	StringToStringMap old = getRequestHeaders();
   	getSettings().setString( REQUEST_HEADERS_PROPERTY, map.toXml() );
   	notifyPropertyChanged( REQUEST_HEADERS_PROPERTY, old, map );
   }
   
   public boolean isInlineResponseAttachments()
   {
   	return getSettings().getBoolean( INLINE_RESPONSE_ATTACHMENTS );
   }
   
   public void setInlineResponseAttachments( boolean inlineResponseAttachments )
   {
   	boolean old = getSettings().getBoolean( INLINE_RESPONSE_ATTACHMENTS );
   	getSettings().setBoolean( INLINE_RESPONSE_ATTACHMENTS, inlineResponseAttachments );
   	notifyPropertyChanged( INLINE_RESPONSE_ATTACHMENTS, old, inlineResponseAttachments );
   }
   
   public boolean isStripWhitespaces()
   {
   	return getSettings().getBoolean( STRIP_WHITESPACES );
   }
   
   public void setStripWhitespaces( boolean stripWhitespaces )
   {
   	boolean old = getSettings().getBoolean( STRIP_WHITESPACES );
   	getSettings().setBoolean( STRIP_WHITESPACES, stripWhitespaces );
   	notifyPropertyChanged( STRIP_WHITESPACES, old, stripWhitespaces );
   }
   
   public boolean isExpandMtomResponseAttachments()
   {
   	return getSettings().getBoolean( EXPAND_MTOM_RESPONSE_ATTACHMENTS );
   }
   
   public void setExpandMtomResponseAttachments( boolean expandMtomResponseAttachments )
   {
   	boolean old = getSettings().getBoolean( EXPAND_MTOM_RESPONSE_ATTACHMENTS );
   	getSettings().setBoolean( EXPAND_MTOM_RESPONSE_ATTACHMENTS, expandMtomResponseAttachments );
   	notifyPropertyChanged( EXPAND_MTOM_RESPONSE_ATTACHMENTS, old, expandMtomResponseAttachments );
   }
   
   /**
    * Use getResponse().getContentAsString();
    * @deprecated
    */
   
   public String getResponseContent()
   {
      return response == null ? null : response.getContentAsString();
   }

   public WsdlResponse getResponse()
   {
   	return response;
   }
   
   public WsdlOperation getOperation()
   {
      return operation;
   }

   public void setRequestContent(String request)
   {
      String old = getRequestContent();
      if( request.equals( old ))
      	return;
      getConfig().setRequest( request );
      definedAttachmentParts = null;
      notifyPropertyChanged( REQUEST_PROPERTY, old, request );
   }
   
   public void setResponse( WsdlResponse response, SubmitContext context )
   {
   	WsdlResponse oldResponse = (WsdlResponse) getResponse();
		this.response = response;
		
      notifyPropertyChanged( RESPONSE_PROPERTY, oldResponse, response );
   }

   public ImageIcon getIcon()
   {
   	return iconAnimator.getIcon();
   }

   public String getUsername()
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getUsername();
   }
   
   public String getPassword()
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getPassword();
   }
   
   public String getDomain()
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getDomain();
   }
   
   public void setUsername( String username )
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = getConfig().addNewCredentials();
   	
   	credentialsConfig.setUsername( username );
   }
   
   public void setPassword( String password )
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = getConfig().addNewCredentials();
   	
   	credentialsConfig.setPassword( password );
   }
   
   public void setDomain( String domain )
   {
   	CredentialsConfig credentialsConfig = getConfig().getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = getConfig().addNewCredentials();
   	
   	credentialsConfig.setDomain( domain );
   }
   
   public void addSubmitListener(SubmitListener listener) 
	{
		listeners.add( listener );
	}

	public void removeSubmitListener(SubmitListener listener) 
	{
		listeners.remove( listener );
	}

	public Submit submit( SubmitContext submitContext, boolean async ) throws SubmitException
	{
      String endpoint = PropertyExpansionRequestFilter.expandProperties( submitContext, getEndpoint());
		if( endpoint == null || endpoint.trim().length() == 0 )
      {
      	UISupport.showErrorMessage( "Missing endpoint for request [" + getName() + "]" );
      	return null;
      }
		
		try
		{
			WsdlSubmit submitter = new WsdlSubmit(this, (SubmitListener[]) listeners.toArray(new SubmitListener[listeners
					.size()]), RequestTransportRegistry.getTransport(endpoint, submitContext));
			submitter.submitRequest(submitContext, async);
			return submitter;
		}
		catch( Exception e )
		{
			throw new SubmitException( e.toString() );
		}
	}
	
	private class InternalInterfaceListener extends InterfaceListenerAdapter implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			if( evt.getPropertyName().equals( Interface.ENDPOINT_PROPERTY ))
			{
				String endpoint = getEndpoint();
				if( evt.getOldValue() != null && evt.getOldValue().equals( endpoint ))
				{
					setEndpoint( (String) evt.getNewValue() );
				}
			}
		}
	}

	public String getWssPasswordType()
	{
		return getConfig().getWssPasswordType();
	}

	public void setWssPasswordType(String wssPasswordType)
	{
		getConfig().setWssPasswordType( wssPasswordType );
	}

	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#attachFile(java.io.File, boolean)
	 */
	public Attachment attachFile( File file, boolean cache )
	{
		try
		{
			FileAttachment fileAttachment = new RequestFileAttachment( file, cache, this );
			attachments.add( fileAttachment );
			notifyPropertyChanged(ATTACHMENTS_PROPERTY, null, fileAttachment );
			return fileAttachment;
		}
		catch (IOException e)
		{
			UISupport.showErrorMessage( e );
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getAttachmentCount()
	 */
	public int getAttachmentCount()
	{
		return attachments.size();
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getAttachmentAt(int)
	 */
	public Attachment getAttachmentAt( int index )
	{
		return attachments.get( index );
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getAttachmentsForPart(java.lang.String)
	 */
	public Attachment [] getAttachmentsForPart( String partName )
	{
		List<Attachment> result = new ArrayList<Attachment>();
		
		for( Attachment attachment : attachments )
		{
			if( attachment.getPart().equals( partName ))
				result.add( attachment );
		}
		
		return result.toArray( new Attachment[result.size()]);
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#removeAttachment(com.eviware.soapui.model.iface.Attachment)
	 */
	public void removeAttachment( Attachment attachment )
	{
		int ix = attachments.indexOf( attachment );
		attachments.remove( ix );
		
		try
		{
			notifyPropertyChanged(ATTACHMENTS_PROPERTY, attachment, null );
		}
		finally
		{
			getConfig().removeAttachment( ix);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getAttachments()
	 */
	public Attachment[] getAttachments()
	{
		return attachments.toArray( new Attachment[attachments.size()] );
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getDefinedAttachmentParts()
	 */
	public WsdlAttachmentPart [] getDefinedAttachmentParts()
	{
		if( definedAttachmentParts == null )
		{
			try
			{
				UISupport.setHourglassCursor();
				definedAttachmentParts = AttachmentUtils.extractAttachmentParts( 
							(WsdlOperation)getOperation(), getRequestContent(), true );
			}
			catch (Exception e)
			{
				log.warn( e.toString() );
			}
			finally 
			{
				UISupport.resetCursor();
			}
		}
		
		return definedAttachmentParts.toArray( new WsdlAttachmentPart[definedAttachmentParts.size()] );
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#getAttachmentPart(java.lang.String)
	 */
	public WsdlAttachmentPart getAttachmentPart( String partName )
	{
		WsdlAttachmentPart[] parts = getDefinedAttachmentParts();
		for( WsdlAttachmentPart part : parts )
		{
			if( part.getName().equals( partName ))
				return part;
		}
		
		return null;
	}

	public void copyAttachmentsTo(WsdlRequest newRequest)
	{
		if( getAttachmentCount() > 0 )
      {
			try
			{
				UISupport.setHourglassCursor();
				for (int c = 0; c < getAttachmentCount(); c++)
				{
					try
					{
						Attachment attachment = getAttachmentAt(c);
						newRequest.addAttachment( attachment );
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}					
				}
			}
			finally 
			{
				UISupport.resetCursor();
			}      	
      }
	}

	private Attachment addAttachment(Attachment attachment)
	{
		if( attachment instanceof FileAttachment )
		{
			AttachmentConfig oldConfig = ((FileAttachment)attachment).getConfig();
			AttachmentConfig newConfig = (AttachmentConfig) getConfig().addNewAttachment().set( oldConfig );
			FileAttachment newAttachment = new RequestFileAttachment( newConfig, this );
			attachments.add( newAttachment);
			return newAttachment;
		}
		else log.error( "Unkown attachment type: " + attachment );
		
		return null;
	}

	public void copyTo(WsdlRequest newRequest)
	{
      newRequest.setEncoding( getEncoding() );
      newRequest.setEndpoint( getEndpoint() );
      newRequest.setRequestContent( getRequestContent() );
      newRequest.setWssPasswordType( getWssPasswordType() );
      
      CredentialsConfig credentials = getConfig().getCredentials();
      if( credentials != null)
      	newRequest.getConfig().setCredentials( (CredentialsConfig) credentials.copy() );

      copyAttachmentsTo( newRequest );
      
      newRequest.setRequestHeaders( getRequestHeaders() );
	}
	
	/* (non-Javadoc)
	 * @see com.eviware.soapui.impl.wsdl.AttachmentContainer#isMtomEnabled()
	 */
	public boolean isMtomEnabled()
	{
		return getSettings().getBoolean( WsdlSettings.ENABLE_MTOM );
	}
	
	public void setMtomEnabled( boolean mtomEnabled )
	{
		getSettings().setBoolean( WsdlSettings.ENABLE_MTOM, mtomEnabled );
		definedAttachmentParts = null;
	}

	public void release()
	{
		super.release();
		
		getOperation().getInterface().removeInterfaceListener( interfaceListener );
		getOperation().getInterface().removePropertyChangeListener( interfaceListener );
	}

	public MessagePart[] getRequestParts()
	{
		try
		{
			// init
			List<MessagePart> result = new ArrayList<MessagePart>();
			WsdlOperation op = (WsdlOperation) getOperation();
			WsdlContext wsdlContext = op.getInterface().getWsdlContext();
			BindingOperation bindingOperation = op.findBindingOperation(wsdlContext.getDefinition());
			
			if( bindingOperation == null )
				return new MessagePart[0];
			
			// header parts
			List<SoapHeader> headers = WsdlUtils.getSoapHeaders( bindingOperation.getBindingInput()
		            .getExtensibilityElements());

	      for (int i = 0; i < headers.size(); i++)
	      {
	      	SoapHeader header = headers.get( i );
	         
	         Message message = wsdlContext.getDefinition().getMessage( header.getMessage() );
	         if( message == null )
	         {
	         	log.error( "Missing message for header: " + header.getMessage() );
	         	continue;
	         }
	         
				javax.wsdl.Part part = message.getPart( header.getPart() );

	         if( part != null )
	         {
	         	SchemaType schemaType = getSchemaTypeForPart( wsdlContext, part );
					if( schemaType != null )
						result.add( new WsdlHeaderPart( part.getName(), schemaType, part.getElementName(), this ));
	         }
	         else
	         	log.error( "Missing part for header; " + header.getPart() );
	      }
	
			// content parts
			javax.wsdl.Part[] parts = WsdlUtils.getInputParts( bindingOperation );
			
			for( int i = 0; i < parts.length; i++ )
	      {
	      	javax.wsdl.Part part = parts[i];
	      	
				if( !WsdlUtils.isAttachmentInputPart( part, bindingOperation ))
	      	{
					SchemaType schemaType = getSchemaTypeForPart( wsdlContext, part );
					if( schemaType != null )
						result.add( new WsdlContentPart( part.getName(), schemaType, part.getElementName(), this ));
      	   }
	      }
			
			result.addAll( Arrays.asList( getDefinedAttachmentParts()) );
			
			return result.toArray( new MessagePart[result.size()] );
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return new MessagePart [0];
		}		
	}
	
	public SchemaType getSchemaTypeForPart( WsdlContext wsdlContext, javax.wsdl.Part part) throws Exception
	{
		SchemaType schemaType = null;
		QName elementName = part.getElementName();
		
		if( elementName != null )
		{
		   SchemaGlobalElement elm = wsdlContext.getSchemaTypeLoader().findElement(elementName);
		   if( elm != null )
		   {
		   	schemaType = elm.getType();
		   }
		   else log.error( "Could not find element [" + elementName + "] specified in part [" + part.getName() + "]" );
		}
		else 
		{
			QName typeName = part.getTypeName();
			
		   if( typeName != null  )
		   {
		      schemaType = wsdlContext.getSchemaTypeLoader().findType( typeName );
		      
		      if( schemaType == null )
		      {
		      	log.error( "Could not find type [" + typeName + "] specified in part [" + part.getName() + "]" );
		      }
		   }
		}
		return schemaType;
	}

	public MessagePart[] getResponseParts()
	{
		try
		{
			// init
			List<MessagePart> result = new ArrayList<MessagePart>();
			WsdlOperation op = (WsdlOperation) getOperation();
			WsdlContext wsdlContext = op.getInterface().getWsdlContext();
			BindingOperation bindingOperation = op.findBindingOperation(wsdlContext.getDefinition());
			
			if( bindingOperation == null )
				return new MessagePart[0];
			
			// header parts
			List<SoapHeader> headers = WsdlUtils.getSoapHeaders( bindingOperation.getBindingOutput()
			            .getExtensibilityElements());

	      for (int i = 0; i < headers.size(); i++)
	      {
	      	SoapHeader header = headers.get( i );
	         
	         Message message = wsdlContext.getDefinition().getMessage( header.getMessage() );
	         if( message == null )
	         {
	         	log.error( "Missing message for header: " + header.getMessage() );
	         	continue;
	         }
	         
				javax.wsdl.Part part = message.getPart( header.getPart() );

	         if( part != null )
	         {
	         	SchemaType schemaType = getSchemaTypeForPart( wsdlContext, part );
					if( schemaType != null )
						result.add( new WsdlHeaderPart( part.getName(), schemaType, part.getElementName(), this ));
	         }
	         else
	         	log.error( "Missing part for header; " + header.getPart() );
	      }
	
			// content parts
			javax.wsdl.Part[] parts = WsdlUtils.getOutputParts( bindingOperation );
			
			for( int i = 0; i < parts.length; i++ )
	      {
	      	javax.wsdl.Part part = parts[i];
	      	
				if( !WsdlUtils.isAttachmentOutputPart( part, bindingOperation ))
	      	{
					SchemaType schemaType = getSchemaTypeForPart( wsdlContext, part );
					if( schemaType != null )
						result.add( new WsdlContentPart( part.getName(), schemaType, part.getElementName(), this ));
      	   }
	      }
			
			result.addAll( Arrays.asList( getDefinedAttachmentParts()) );
			
			return result.toArray( new MessagePart[result.size()] );
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return new MessagePart [0];
		}		
	}
	
	protected class RequestIconAnimator extends ModelItemIconAnimator implements SubmitListener
	{
		public RequestIconAnimator()
		{
			super( WsdlRequest.this, "/request.gif", 
						new String[] {"/exec_request_1.gif", "/exec_request_2.gif",
						"/exec_request_3.gif", "/exec_request_4.gif"} );
		}
		
		public boolean beforeSubmit(Submit submit, SubmitContext context) 
		{
			if( isEnabled() && submit.getRequest() == getTarget() )
				start();
	      return true;
		}

		public void afterSubmit(Submit submit, SubmitContext context) 
		{
			if( submit.getRequest() == getTarget() )
				stop();
		}
	}

	public boolean isMultipartEnabled()
	{
		return !getSettings().getBoolean( DISABLE_MULTIPART_ATTACHMENTS );
	}
	
	public void setMultipartEnabled( boolean multipartEnabled )
	{
		getSettings().setBoolean( DISABLE_MULTIPART_ATTACHMENTS, multipartEnabled );
	}
}
