/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.mock;

import groovy.lang.Binding;
import groovy.lang.Script;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.DispatchStyleConfig;
import com.eviware.soapui.config.MockOperationConfig;
import com.eviware.soapui.config.MockResponseConfig;
import com.eviware.soapui.config.DispatchStyleConfig.Enum;
import com.eviware.soapui.impl.actions.ShowDesktopPanelAction;
import com.eviware.soapui.impl.wsdl.AbstractWsdlModelItem;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.actions.mockoperation.NewMockResponseAction;
import com.eviware.soapui.impl.wsdl.actions.mockoperation.RemoveMockOperationAction;
import com.eviware.soapui.impl.wsdl.actions.mockoperation.RenameMockOperationAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.OpenRequestForMockOperationAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.mock.MockOperation;
import com.eviware.soapui.model.mock.MockResponse;
import com.eviware.soapui.model.mock.MockRunContext;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.xml.XmlUtils;

public class WsdlMockOperation extends AbstractWsdlModelItem<MockOperationConfig> implements MockOperation, PropertyChangeListener
{
	public final static String DISPATCH_STYLE_PROPERTY = WsdlMockOperation.class.getName() + "@dispatchstyle";
	public final static String DEFAULT_RESPONSE_PROPERTY = WsdlMockOperation.class.getName() + "@defaultresponse";
	public final static String DISPATCH_PATH_PROPERTY = WsdlMockOperation.class.getName() + "@dispatchpath";
	
	private WsdlOperation operation;
	private List<WsdlMockResponse> responses = new ArrayList<WsdlMockResponse>();
	private int currentDispatchIndex;
	
	public WsdlMockOperation(WsdlMockService mockService, MockOperationConfig config)
   {
   	super( config, mockService, "/mockOperation.gif" );
   	
   	Interface iface = mockService.getProject().getInterfaceByName( config.getInterface() );
   	operation = ( WsdlOperation ) iface.getOperationByName( config.getOperation() );
   	
   	List<MockResponseConfig> responseConfigs = config.getResponseList();
   	for( MockResponseConfig responseConfig : responseConfigs )
   	{
   		WsdlMockResponse wsdlMockResponse = new WsdlMockResponse( this, responseConfig );
   		wsdlMockResponse.addPropertyChangeListener( this );
			responses.add( wsdlMockResponse );
   	}
   	
   	initData( config );
   	
   	addActions();
   }

	private void initData( MockOperationConfig config )
	{
		if( !config.isSetName() )
   		config.setName( operation.getName() );
   	
   	if( !config.isSetDispatchStyle())
   		config.setDispatchStyle( DispatchStyleConfig.SEQUENCE );
   	
   	if( !config.isSetDefaultResponse() && responses.size() > 0 )
   		setDefaultResponse( responses.get( 0 ).getName() );
	}
	
	private void addActions()
	{
		addAction( new ShowDesktopPanelAction( "Open MockOperation Editor", 
         		"Opens the MockOperation Editor for this MockOperation", this ));
		addAction( ActionSupport.SEPARATOR_ACTION );
		addAction( new OpenRequestForMockOperationAction( this));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new NewMockResponseAction( this ));
      addAction( new RenameMockOperationAction( this ) );
      addAction( new RemoveMockOperationAction( this ) );
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.MOCKOPERATION_HELP_URL ));
	}
	
	public WsdlMockOperation( WsdlMockService mockService, MockOperationConfig config, WsdlOperation operation )
	{
		super( config, mockService, "/mockOperation.gif" );
		this.operation = operation;
		
		config.setInterface( operation.getInterface().getName() );
		config.setOperation( operation.getName() );
		
		initData( config );
		addActions();
	}

	public WsdlMockService getMockService()
	{
		return ( WsdlMockService ) getParent();
	}

	public WsdlMockResponse getMockResponseAt( int index )
	{
		return responses.get( index );
	}

	public WsdlOperation getOperation()
	{
		return operation;
	}

	public WsdlMockResponse getMockResponseByName( String name )
	{
		return ( WsdlMockResponse ) getWsdlModelItemByName( responses, name );
	}

	public int getMockResponseCount()
	{
		return responses.size();
	}

	public WsdlMockResponse addNewMockResponse( MockResponseConfig responseConfig )
	{
		WsdlMockResponse mockResponse = new WsdlMockResponse( this, responseConfig );
		
		responses.add( mockResponse );
		if( responses.size() == 1 )
			setDefaultResponse( mockResponse.getName() );
		
		((WsdlMockService)getMockService()).fireMockResponseAdded( mockResponse );
		
		return mockResponse;
	}
	
	public WsdlMockResponse addNewMockResponse( String name, boolean createResponse )
	{
		MockResponseConfig responseConfig = getConfig().addNewResponse();
		responseConfig.setName( name );
		
		if( createResponse && getOperation() != null )
		{
			boolean createOptional = SoapUI.getSettings().getBoolean( WsdlSettings.XML_GENERATION_ALWAYS_INCLUDE_OPTIONAL_ELEMENTS );
			responseConfig.setResponseContent( getOperation().createResponse( createOptional ));
		}
		
		return addNewMockResponse( responseConfig );
	}
	
	public void removeMockResponse( WsdlMockResponse mockResponse )
   {
      int ix = responses.indexOf( mockResponse );
      responses.remove( ix );
      mockResponse.removePropertyChangeListener( this );
      
      try
      {
      	((WsdlMockService)getMockService()).fireMockResponseRemoved( mockResponse );
      }
      finally
      {
	      mockResponse.release();
	      getConfig().removeResponse( ix );
      }
   }

	public WsdlMockResult dispatchRequest( WsdlMockRequest request, HttpServletResponse response ) throws DispatchException
	{
		try
		{
			WsdlMockResult result = new WsdlMockResult( request, response );
			
			if( getMockResponseCount() == 0 )
				throw new DispatchException( "Missing MockResponse(s) in MockOperation [" + getName() + "]" );
			
			if( getDispatchStyle() == DispatchStyleConfig.XPATH )
			{
				XmlObject[] items = evaluateDispatchXPath( request );
				for( XmlObject item : items )
				{
					WsdlMockResponse mockResponse = getMockResponseByName( XmlUtils.getNodeValue( item.getDomNode() ));
					
					if( mockResponse == null )
						mockResponse = getMockResponseByName( getDefaultResponse() );
					
					if( mockResponse != null )
					{
						result.setMockResponse( mockResponse );
						mockResponse.execute( request, result );
						
						return result;
					}
				}
				
				throw new DispatchException( "Missing matching response message" );
			}
			else if( getDispatchStyle() == DispatchStyleConfig.SCRIPT )
			{
				Object retVal = evaluateDispatchScript( request );
				
				WsdlMockResponse mockResponse = retVal == null ? getMockResponseByName( getDefaultResponse() ) 
							: getMockResponseByName( retVal.toString() );
				
				if( mockResponse != null )
				{
					result.setMockResponse( mockResponse );
					mockResponse.execute( request, result );
					
					return result;
				}
				else
				{
					throw new DispatchException( "Missing matching response message [" + retVal + "]" );
				}
			}
			else 
			{
				WsdlMockResponse mockResponse = null;
				synchronized( this )
				{
					if( getDispatchStyle() == DispatchStyleConfig.RANDOM )
					{
						currentDispatchIndex = ( int ) ( (Math.random() * getMockResponseCount()) + 0.5F );
					}
	
					if( currentDispatchIndex >= getMockResponseCount() ) 
						currentDispatchIndex = 0;
	
					mockResponse = getMockResponseAt( currentDispatchIndex );
					result.setMockResponse( mockResponse );
					
					currentDispatchIndex++;
				}

				mockResponse.execute( request, result );
			}
			
			return result;
		}
		catch( Exception e )
		{
			throw new DispatchException( e );
		}
	}

	public XmlObject[] evaluateDispatchXPath( WsdlMockRequest request ) throws XmlException
	{
		XmlObject xmlObject = request.getRequestXmlObject();
		XmlObject[] items = xmlObject.selectPath( getDispatchPath() );
		return items;
	}

	public Object evaluateDispatchScript( WsdlMockRequest request ) throws DispatchException
	{
		String dispatchPath = getDispatchPath();
		if( dispatchPath == null || dispatchPath.trim().length() == 0 )
		{
			throw new DispatchException( "Dispatch Script is empty" );
		}
		
		try
		{
			WsdlMockService mockService = getMockService();
			WsdlMockRunner mockRunner = mockService.getMockRunner();
			MockRunContext context = mockRunner == null ? new WsdlMockRunContext( mockService ) : mockRunner.getMockContext(); 
			
			Binding localBinding = new Binding();
			localBinding.setVariable( "context", context );
			localBinding.setVariable( "mockRequest", request);
			localBinding.setVariable( "mockOperation", this );
			localBinding.setVariable( "log", SoapUI.ensureGroovyLog() );
			
			Script localScript = getMockService().getScriptShell().parse( dispatchPath );
			localScript.setBinding( localBinding );
			Object retVal = localScript.run();
			return retVal;
		}
		catch( Throwable e )
		{
			e.printStackTrace();
			throw new DispatchException( "Failed to dispatch using script; " + e );
		}
	}
	
	public DispatchStyleConfig.Enum getDispatchStyle()
	{
		return getConfig().getDispatchStyle();
	}
	
	public void setDispatchStyle( DispatchStyleConfig.Enum dispatchStyle )
	{
		Enum old = getDispatchStyle();
		getConfig().setDispatchStyle( dispatchStyle );
		notifyPropertyChanged( DISPATCH_STYLE_PROPERTY, old, dispatchStyle );
	}
	
	public String getDispatchPath()
	{
		return getConfig().getDispatchPath();
	}
	
	public void setDispatchPath( String dispatchPath )
	{
		String old = getDispatchPath();
		getConfig().setDispatchPath( dispatchPath );
		notifyPropertyChanged( DISPATCH_PATH_PROPERTY, old, dispatchPath );
	}
	
	public String getWsdlOperationName()
	{
		return operation == null ? null : operation.getName();
	}

	public String getDefaultResponse()
	{
		return getConfig().getDefaultResponse();
	}
	
	public void setDefaultResponse( String defaultResponse )
	{
		String old = getDefaultResponse();
		getConfig().setDefaultResponse( defaultResponse );
		notifyPropertyChanged( DEFAULT_RESPONSE_PROPERTY, old, defaultResponse );
	}

	public List<MockResponse> getMockResponses()
	{
		return new ArrayList<MockResponse>( responses );
	}

	public void propertyChange( PropertyChangeEvent arg0 )
	{
		if( arg0.getPropertyName().equals( WsdlMockResponse.NAME_PROPERTY ))
		{
			if( arg0.getOldValue().equals( getDefaultResponse() ))
				setDefaultResponse( arg0.getNewValue().toString() );
		}
	}
	
	public WsdlMockResult getLastMockResult()
	{
		WsdlMockResult result = null;
		
		for( WsdlMockResponse response : responses )
		{
			WsdlMockResult mockResult = response.getMockResult();
			if( mockResult != null )
			{
				if( result == null || result.getTimestamp() > mockResult.getTimestamp() )
					result = mockResult;
			}
		}
		
		return result;
	}

	public void setOperation( WsdlOperation operation )
	{
		if( operation == null )
		{
			getConfig().unsetInterface();
			getConfig().unsetOperation();
		}
		else
		{
			getConfig().setInterface( operation.getInterface().getName() );
			getConfig().setOperation( operation.getName() );
		}
		
		this.operation = operation;
	}
}
