/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.impl.wsdl.actions.testcase.AddNewLoadTestAction;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.LoadTest;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuiteListener;
import com.eviware.soapui.model.tree.AbstractModelItemTreeNode;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeModel;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * SoapUITreeNode for TestCase implementations
 * 
 * @author Ole.Matzura
 */

public class TestCaseTreeNode  extends AbstractModelItemTreeNode<TestCase>
{
	private TestStepsTreeNode testStepsNode;
	private LoadTestsTreeNode loadTestsNode;
	private InternalTestSuiteListener internalTestSuiteListener;
	
	public TestCaseTreeNode( TestCase testCase, SoapUITreeModel treeModel )
	{
		super( testCase, testCase.getTestSuite(), treeModel );

		testStepsNode = new TestStepsTreeNode();
		loadTestsNode = new LoadTestsTreeNode();
		
		getTreeModel().mapModelItem( testStepsNode );
		getTreeModel().mapModelItem( loadTestsNode );
		
		internalTestSuiteListener = new InternalTestSuiteListener();
		testCase.getTestSuite().addTestSuiteListener( internalTestSuiteListener );
	}

	public void release()
	{
		super.release();

		getTestCase().getTestSuite().removeTestSuiteListener( internalTestSuiteListener );
		
		testStepsNode.release();
		loadTestsNode.release();
	}

	public int getChildCount()
	{
		return 2;
	}

	public SoapUITreeNode getChildNode(int index)
	{
		if( index == 0 )
			return testStepsNode;
		
		if( index == 1 )
			return loadTestsNode;
		
		return null;
	}

	public int getIndexOfChild(Object child)
	{
		if( child == testStepsNode ) return 0;
		if( child == loadTestsNode ) return 1;
		
		return -1;
	}
	
	public LoadTestsTreeNode getLoadTestsNode()
	{
		return loadTestsNode;
	}

	public TestStepsTreeNode getTestStepsNode()
	{
		return testStepsNode;
	}

	public TestCase getTestCase()
	{
		return (TestCase) getModelItem();
	}
   
   public class TestStepsTreeNode extends AbstractTreeNode
	{
   	private List<TestStepTreeNode> testStepNodes = new ArrayList<TestStepTreeNode>();
		
   	
		protected TestStepsTreeNode()
		{
			super( "Test Steps (" + getTestCase().getTestStepCount() + ")", UISupport.createImageIcon("/teststeps.gif"));
			
			for( int c = 0; c < getTestCase().getTestStepCount(); c++ )
			{
				testStepNodes.add( new TestStepTreeNode( getTestCase().getTestStepAt( c ), 
						getModelItem(), getTreeModel() ));
			}
			
			getTreeModel().mapModelItems( testStepNodes );

			addAction( getTestCase().getActions().getDefaultAction(), true );
			addAction( new ActionSupport.ActionListAction( ((WsdlTestCase)getTestCase()).getCreateActions() ), false);
		}

		public void updateLabel()
		{
			setLabel( "Test Steps (" + getTestCase().getTestStepCount() + ")" ); 
		}
		
		public int getChildCount()
		{
			return testStepNodes.size();
		}

		public int getIndexOfChild(Object child)
		{
			return testStepNodes.indexOf( child );
		}

		public SoapUITreeNode getChildNode(int index)
		{
			return testStepNodes.get( index );
		}

		public SoapUITreeNode getParentTreeNode()
		{
			return TestCaseTreeNode.this;
		}

		public void testStepInserted(TestStep testStep, int index)
		{
			TestStepTreeNode testStepTreeNode = new TestStepTreeNode( testStep, 
					getModelItem(), getTreeModel() );
			testStepNodes.add( index, testStepTreeNode);
			getTreeModel().notifyNodeInserted( testStepTreeNode );
		}

		public void testStepRemoved(TestStep testStep, int index)
		{
			SoapUITreeNode treeNode = getTreeModel().getTreeNode( testStep );
      	if( testStepNodes.contains( treeNode ))
      	{
      		getTreeModel().notifyNodeRemoved( treeNode );
      		testStepNodes.remove( treeNode );
      	}
      	else throw new RuntimeException( "Removing unkown testStep" );
		}

		public void testStepMoved(TestStep testStep, int fromIndex, int offset)
		{
			testStepRemoved( testStep, fromIndex );
			testStepInserted( testStep, fromIndex+offset );
		}

		public void release()
		{
			for( TestStepTreeNode testStepNode : testStepNodes )
				testStepNode.release();
		}
	}
	
	public class LoadTestsTreeNode extends AbstractTreeNode
	{
		private List<LoadTestTreeNode> loadTestNodes = new ArrayList<LoadTestTreeNode>();
		
		protected LoadTestsTreeNode()
		{
			super( "Load Tests (" + getTestCase().getLoadTestCount() + ")", UISupport.createImageIcon( "/loadtests.gif") );
			
			for( int c = 0; c < getTestCase().getLoadTestCount(); c++ )
			{
				loadTestNodes.add( new LoadTestTreeNode( getTestCase().getLoadTestAt( c ), 
						getModelItem(), getTreeModel() ));
			}
			
			getTreeModel().mapModelItems( loadTestNodes );
			
			addAction( new AddNewLoadTestAction( (WsdlTestCase) getTestCase() ), false );
		}

		public int getChildCount()
		{
			return loadTestNodes.size();
		}

		public int getIndexOfChild(Object child)
		{
			return loadTestNodes.indexOf( child );
		}

		public SoapUITreeNode getChildNode(int index)
		{
			return loadTestNodes.get( index );
		}

		public SoapUITreeNode getParentTreeNode()
		{
			return TestCaseTreeNode.this;
		}

		public void loadTestInserted(LoadTest loadTest)
		{
			LoadTestTreeNode loadTestTreeNode = new LoadTestTreeNode( loadTest, 
					getModelItem(), getTreeModel() );
			loadTestNodes.add( loadTestTreeNode);
			getTreeModel().notifyNodeInserted( loadTestTreeNode );
		}

		public void loadTestRemoved(LoadTest loadTest)
		{
			SoapUITreeNode treeNode = getTreeModel().getTreeNode( loadTest );
      	if( loadTestNodes.contains( treeNode ))
      	{
      		getTreeModel().notifyNodeRemoved( treeNode );
      		loadTestNodes.remove( treeNode);
      	}
      	else throw new RuntimeException( "Removing unkown loadTest" );
      }
	
		public void release()
		{
			for( LoadTestTreeNode loadTestNode : loadTestNodes )
				loadTestNode.release();
		}

		public void updateLabel()
		{
			setLabel( "Load Tests (" + getTestCase().getLoadTestCount() + ")" ); 
		}
	}

	public void testStepInserted(TestStep testStep, int index)
	{
		testStepsNode.testStepInserted( testStep, index );
	}

	public void testStepRemoved(TestStep testStep, int index)
	{
		testStepsNode.testStepRemoved( testStep, index );
	}

	public void loadTestInserted(LoadTest loadTest)
	{
		loadTestsNode.loadTestInserted( loadTest );
	}

	public void loadTestRemoved(LoadTest loadTest)
	{
		loadTestsNode.loadTestRemoved( loadTest );
	}

	public void testStepMoved(TestStep testStep, int fromIndex, int offset)
	{
		testStepsNode.testStepMoved( testStep, fromIndex, offset );
	}
	
	public class InternalTestSuiteListener extends TestSuiteListenerAdapter implements TestSuiteListener
	{
		@Override
		public void testStepAdded( TestStep testStep, int index )
		{
			testStepsNode.updateLabel();
		}
		
		@Override
		public void testStepRemoved( TestStep testStep, int index )
		{
			testStepsNode.updateLabel();
		}

		@Override
		public void loadTestAdded( LoadTest loadTest )
		{
			loadTestsNode.updateLabel();
		}

		@Override
		public void loadTestRemoved( LoadTest loadTest )
		{
			loadTestsNode.updateLabel();
		}
		
		
	}
}
