/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.tools;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.model.mock.MockResult;
import com.eviware.soapui.model.mock.MockRunListener;
import com.eviware.soapui.model.mock.MockRunner;
import com.eviware.soapui.model.mock.MockService;

/**
 * Standalone tool-runner used from maven-plugin, can also be used from command-line (see xdocs) or
 * directly from other classes.
 * <p>
 * For standalone usage, set the project file (with setProjectFile) and other desired properties before
 * calling run</p> 
 * 
 * @author Ole.Matzura
 */

public class SoapUIMockServiceRunner
{
	private final static Logger log = Logger.getLogger( SoapUIMockServiceRunner.class );
	
	private String projectFile;
	private String mockService;
	private String port;
	private String path;
	
	public static String TITLE = "soapUI " + SoapUI.SOAPUI_VERSION + " MockService Runner";

	/**
	 * Runs the specified MockService in the specified soapUI project file, see soapUI xdocs for details.
	 * 
	 * @param args
	 * @throws Exception
	 */

	@SuppressWarnings("static-access")
	public static void main( String [] args) throws Exception
	{
		System.out.println( TITLE );
		SoapUIMockServiceRunner runner = new SoapUIMockServiceRunner();
		
		Options options = new Options();
		options.addOption( "m", true, "Sets the MockService" );
		options.addOption( "p", true, "Sets the local port to listen on" );
		options.addOption( "a", true, "Sets the url path to listen on" );
		options.addOption( "s", false, "Sets the soapui-settings.xml file to use" );
		
		CommandLineParser parser = new PosixParser();
		CommandLine cmd = parser.parse( options, args);
		
		String[] args2 = cmd.getArgs();
		if( args2.length != 1 || !cmd.hasOption( "m") )
		{
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp( "mockservicerunner [options] <soapui-project-file>", options );

			System.err.println( "Missing soapUI project file and/or MockService.." );
			return;
		}
		
		runner.setProjectFile( args2[0] );
		runner.setMockService( cmd.getOptionValue( "m") );
		
		if( cmd.hasOption( "a"))
			runner.setPath( cmd.getOptionValue( "a" ) );

		if( cmd.hasOption( "p"))
			runner.setPort( cmd.getOptionValue( "p" ) );
		
		if( cmd.hasOption( "s"))
			SoapUI.initSettings( cmd.getOptionValue( "s" ));

		try
		{
			SoapUI.loadExtLibs();
			
			runner.run();
			System.exit( 0 );
		}
		catch (Exception e)
		{
			log.error( e.toString() );
			e.printStackTrace();
			System.exit( 1 );
		}		
	}

	public void setProjectFile(String projectFile)
	{
		this.projectFile = projectFile;
	}

	public void setMockService(String mockService)
	{
		this.mockService = mockService;
	}

	public void setPath( String path )
	{
		this.path = path;
	}

	public void setPort( String port )
	{
		this.port = port;
	}

	public SoapUIMockServiceRunner()
	{
	}
	
	public void run() throws Exception
	{
		if( !new File( projectFile ).exists() )
			throw new Exception( "soapUI project file [" + projectFile + "] not found" );
		
		WsdlProject project = new WsdlProject( projectFile, null );
		log.info( "Running MockService [" + mockService + "] in project [" + project.getName() + "]" );
		log.info( "Press any key to terminate" );
		
		long startTime = System.nanoTime();
		
		for( int c = 0; c < project.getMockServiceCount(); c++ )
		{
			MockService ms = project.getMockServiceAt( c );
			if( ms.getName().equals( mockService ))
				runMockService( ( WsdlMockService ) ms );
		}
		
		long timeTaken = (System.nanoTime()-startTime)/1000000;
		log.info( "time taken: " + timeTaken + "ms" );
	}
	
	/**
	 * Runs the configured tool for the specified interface.. needs to be refactored to use
	 * some kind of registry/factory pattern for tools
	 * 
	 * @param iface
	 */
	
	public void runMockService( WsdlMockService mockService )
	{
		try
		{
			if( path != null )
				mockService.setPath( path );
			
			if( port != null )
				mockService.setPort( Integer.parseInt( port ));
			
			mockService.addMockRunListener( new LogListener() );
			MockRunner runner = mockService.start();
			
			System.in.read();
			runner.stop();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void log(String msg)
	{
		System.out.print( msg );
	}

   public void logError( String msg )
   {
      System.err.println( msg );
   }

   public class LogListener implements MockRunListener
	{
   	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		private int responseCount;
   	
		public void onMockRunnerStart( MockRunner mockRunner )
		{
			log.info( "MockService started on port " + mockRunner.getMockService().getPort() + " at path [" + 
						mockRunner.getMockService().getPath() + "]" );
		}

		public void onMockRunnerStop( MockRunner mockRunner )
		{
			log.info( "MockService stopped, handled " + responseCount + " requests" );
		}

		public void onMockResult( MockResult result )
		{
			responseCount++;
			log.info( "Handled request " + responseCount + "; [" + result.getMockResponse().getMockOperation().getName() + 
						"] with [" + result.getMockResponse().getName() + "] in [" + result.getTimeTaken() + 
						"ms] at [" + dateFormat.format( new Date( result.getTimestamp())) + "]" );
		}

		public void onMockRequest( MockRunner runner, HttpServletRequest request, HttpServletResponse response )
		{
			// TODO Auto-generated method stub
			
		}
	}
}